/****************************************************************************
#	 	Gideo: A Small (spca5x) Video Capture utility                       #
# 		Copyright (C) 2004 David Truman	                                    #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 2 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program; if not, write to the Free Software               #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA #
#                                                                           #
****************************************************************************/



#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include <bonobo.h>
#include <gnome.h> 

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "globals.h"

unsigned char pixname[1024];
unsigned char title[255];

#define GLADE_HOOKUP_OBJECT(component,widget,name) \
  g_object_set_data_full (G_OBJECT (component), name, \
    gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)

#define GLADE_HOOKUP_OBJECT_NO_REF(component,widget,name) \
  g_object_set_data (G_OBJECT (component), name, widget)

static GnomeUIInfo menu2_uiinfo[] =
{
  {
    GNOME_APP_UI_ITEM, N_("JPG"),
    NULL,
    (gpointer) on_jpg1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("YUV"),
    NULL,
    (gpointer) on_yuv1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("R16"),
    NULL,
    (gpointer) on_r1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("R24"),
    NULL,
    (gpointer) on_r2_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("R32"),
    NULL,
    (gpointer) on_r3_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_END
};

static GnomeUIInfo menu1_uiinfo[] =
{
  {
    GNOME_APP_UI_ITEM, N_("VGA"),
    NULL,
    (gpointer) on_vga1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("PAL"),
    NULL,
    (gpointer) on_pal1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("SIF"),
    NULL,
    (gpointer) on_sif1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("CIF"),
    NULL,
    (gpointer) on_cif1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("QPAL"),
    NULL,
    (gpointer) on_qpal1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("QSIF"),
    NULL,
    (gpointer) on_qsif1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("QCIF"),
    NULL,
    (gpointer) on_qcif1_activate, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_END
};

char *fullpixname(char *endbit)
{
     memset(&pixname,0,1024);
	 sprintf((char *)&pixname,"%s/%s",PACKAGE_PIXMAPS_DIR,endbit);	
	 return (char *)&pixname;
}

void hidemenus(gboolean bl)
{
  if (bl){
	  gtk_widget_hide(GTK_WIDGET(menu1_uiinfo[1].widget));
	  gtk_widget_hide(GTK_WIDGET(menu1_uiinfo[2].widget));
	  gtk_widget_hide(GTK_WIDGET(menu1_uiinfo[4].widget));
	  gtk_widget_hide(GTK_WIDGET(menu1_uiinfo[5].widget));
	  gtk_widget_hide(GTK_WIDGET(menu1_uiinfo[6].widget));
  }
  else{
	  gtk_widget_show(GTK_WIDGET(menu1_uiinfo[1].widget));
	  gtk_widget_show(GTK_WIDGET(menu1_uiinfo[2].widget));
	  gtk_widget_show(GTK_WIDGET(menu1_uiinfo[4].widget));
	  gtk_widget_show(GTK_WIDGET(menu1_uiinfo[5].widget));
	  /* Never Seem to Work on Mine so takenout */
	  //gtk_widget_show(GTK_WIDGET(menu1_uiinfo[6].widget));
  }
	
}

GtkWidget*
create_window1 (void)
{
  GtkWidget *window1;
  GtkWidget *fixed1;
  GtkWidget *label1;
  GtkWidget *label2;
  GtkWidget *label3;
  GtkWidget *optionmenu2;
  GtkWidget *menu2;
  GtkWidget *menu1;
  GtkWidget *hscale1;
  GtkWidget *hscale2;
  GtkWidget *hscale3;
  GtkWidget *togglebutton2;
  GtkWidget *image4;
  GtkWidget *togglebutton1;
  GtkWidget *image3;
  GtkWidget *togglebutton3;
  GtkWidget *image5;
  GtkWidget *togglebutton4;
  GtkWidget *image6;
  GtkWidget *togglebutton5;
  GtkWidget *image7;

  window1 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  memset((char*)&title,0,255);
  sprintf((char*)&title,"%s v%s",PACKAGE,(char *)VERSION);
  gtk_window_set_title (GTK_WINDOW (window1), _((char*)title));
  gtk_window_set_default_size (GTK_WINDOW (window1), 150, 198);

  fixed1 = gtk_fixed_new ();
  gtk_widget_show (fixed1);
  gtk_container_add (GTK_CONTAINER (window1), fixed1);

  label1 = gtk_label_new (_("Brightness"));
  gtk_widget_show (label1);
  gtk_fixed_put (GTK_FIXED (fixed1), label1, 32, 36);
  gtk_widget_set_size_request (label1, 84, 16);

  label2 = gtk_label_new (_("Contrast"));
  gtk_widget_show (label2);
  gtk_fixed_put (GTK_FIXED (fixed1), label2, 32, 76);
  gtk_widget_set_size_request (label2, 84, 16);

  label3 = gtk_label_new (_("Colors"));
  gtk_widget_show (label3);
  gtk_fixed_put (GTK_FIXED (fixed1), label3, 32, 116);
  gtk_widget_set_size_request (label3, 84, 16);

  optionmenu2 = gtk_option_menu_new ();
  gtk_widget_show (optionmenu2);
  gtk_fixed_put (GTK_FIXED (fixed1), optionmenu2, 76, 164);
  gtk_widget_set_size_request (optionmenu2, 68, 28);

  menu2 = gtk_menu_new ();
  gnome_app_fill_menu (GTK_MENU_SHELL (menu2), menu2_uiinfo,
                       NULL, FALSE, 0);


  gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu2), menu2);

  optionmenu1 = gtk_option_menu_new ();
  gtk_widget_show (optionmenu1);
  gtk_fixed_put (GTK_FIXED (fixed1), optionmenu1, 4, 164);
  gtk_widget_set_size_request (optionmenu1, 68, 28);

  menu1 = gtk_menu_new ();
  gnome_app_fill_menu (GTK_MENU_SHELL (menu1), menu1_uiinfo,
                       NULL, FALSE, 0);

  gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu1), menu1);

  hscale1 = gtk_hscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (128, 0, 255, 1, 1, 1)));
  gtk_widget_show (hscale1);
  gtk_fixed_put (GTK_FIXED (fixed1), hscale1, 4, 48);
  gtk_widget_set_size_request (hscale1, 140, 36);
  gtk_scale_set_draw_value (GTK_SCALE (hscale1), FALSE);

  hscale2 = gtk_hscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (32, 0, 255, 1, 1, 0)));
  gtk_widget_show (hscale2);
  gtk_fixed_put (GTK_FIXED (fixed1), hscale2, 4, 88);
  gtk_widget_set_size_request (hscale2, 140, 36);
  gtk_scale_set_draw_value (GTK_SCALE (hscale2), FALSE);

  hscale3 = gtk_hscale_new (GTK_ADJUSTMENT (gtk_adjustment_new (32, 0, 255, 1, 1, 1)));
  gtk_widget_show (hscale3);
  gtk_fixed_put (GTK_FIXED (fixed1), hscale3, 4, 128);
  gtk_widget_set_size_request (hscale3, 140, 36);
  gtk_scale_set_draw_value (GTK_SCALE (hscale3), FALSE);

  togglebutton2 = gtk_toggle_button_new ();
  gtk_widget_show (togglebutton2);
  gtk_fixed_put (GTK_FIXED (fixed1), togglebutton2, 32, 4);
  gtk_widget_set_size_request (togglebutton2, 28, 24);

  image4 = create_pixmap (window1, fullpixname("film.png"));
  gtk_widget_show (image4);
  gtk_container_add (GTK_CONTAINER (togglebutton2), image4);

  togglebutton1 = gtk_toggle_button_new ();
  gtk_widget_show (togglebutton1);
  gtk_fixed_put (GTK_FIXED (fixed1), togglebutton1, 4, 4);
  gtk_widget_set_size_request (togglebutton1, 28, 24);

  image3 = create_pixmap (window1, fullpixname("snap.png"));
  gtk_widget_show (image3);
  gtk_container_add (GTK_CONTAINER (togglebutton1), image3);

  togglebutton3 = gtk_toggle_button_new ();
  gtk_widget_show (togglebutton3);
  gtk_fixed_put (GTK_FIXED (fixed1), togglebutton3, 60, 4);
  gtk_widget_set_size_request (togglebutton3, 28, 24);

  image5 = create_pixmap (window1, fullpixname	("pause.png"));
  gtk_widget_show (image5);
  gtk_container_add (GTK_CONTAINER (togglebutton3), image5);

  togglebutton4 = gtk_toggle_button_new ();
  gtk_widget_show (togglebutton4);
  gtk_fixed_put (GTK_FIXED (fixed1), togglebutton4, 88, 4);
  gtk_widget_set_size_request (togglebutton4, 28, 24);

  image6 = create_pixmap (window1, fullpixname("invert.png"));
  gtk_widget_show (image6);
  gtk_container_add (GTK_CONTAINER (togglebutton4), image6);

  togglebutton5 = gtk_toggle_button_new ();
  gtk_widget_show (togglebutton5);
  gtk_fixed_put (GTK_FIXED (fixed1), togglebutton5, 116, 4);
  gtk_widget_set_size_request (togglebutton5, 28, 24);

  image7 = create_pixmap (window1, fullpixname("close.png"));
  gtk_widget_show (image7);
  gtk_container_add (GTK_CONTAINER (togglebutton5), image7);



  g_signal_connect ((gpointer) window1, "delete_event",
                    G_CALLBACK (on_window1_delete_event),
                    NULL);
  g_signal_connect ((gpointer) window1, "destroy_event",
                    G_CALLBACK (on_window1_destroy_event),
                    NULL);
  g_signal_connect ((gpointer) optionmenu2, "changed",
                    G_CALLBACK (on_optionmenu2_changed),
                    NULL);
  g_signal_connect ((gpointer) optionmenu1, "changed",
                    G_CALLBACK (on_optionmenu1_changed),
                    NULL);
  g_signal_connect ((gpointer) hscale1, "value_changed",
                    G_CALLBACK (on_hscale1_value_changed),
                    NULL);
  g_signal_connect ((gpointer) hscale2, "value_changed",
                    G_CALLBACK (on_hscale2_value_changed),
                    NULL);
  g_signal_connect ((gpointer) hscale3, "value_changed",
                    G_CALLBACK (on_hscale3_value_changed),
                    NULL);
  g_signal_connect ((gpointer) togglebutton2, "toggled",
                    G_CALLBACK (on_togglebutton2_toggled),
                    NULL);
  g_signal_connect ((gpointer) togglebutton1, "toggled",
                    G_CALLBACK (on_togglebutton1_toggled),
                    NULL);
  g_signal_connect ((gpointer) togglebutton3, "toggled",
                    G_CALLBACK (on_togglebutton3_toggled),
                    NULL);
  g_signal_connect ((gpointer) togglebutton4, "toggled",
                    G_CALLBACK (on_togglebutton4_toggled),
                    NULL);
  g_signal_connect ((gpointer) togglebutton5, "toggled",
                    G_CALLBACK (on_togglebutton5_toggled),
                    NULL);




  /* Store pointers to all widgets, for use by lookup_widget(). */
  GLADE_HOOKUP_OBJECT_NO_REF (window1, window1, "window1");
  GLADE_HOOKUP_OBJECT (window1, fixed1, "fixed1");
  GLADE_HOOKUP_OBJECT (window1, label1, "label1");
  GLADE_HOOKUP_OBJECT (window1, label2, "label2");
  GLADE_HOOKUP_OBJECT (window1, label3, "label3");
  GLADE_HOOKUP_OBJECT (window1, optionmenu2, "optionmenu2");
  GLADE_HOOKUP_OBJECT (window1, menu2, "menu2");
  GLADE_HOOKUP_OBJECT (window1, menu2_uiinfo[0].widget, "jpg1");
  GLADE_HOOKUP_OBJECT (window1, menu2_uiinfo[1].widget, "yuv1");
  GLADE_HOOKUP_OBJECT (window1, menu2_uiinfo[2].widget, "r1");
  GLADE_HOOKUP_OBJECT (window1, menu2_uiinfo[3].widget, "r2");
  GLADE_HOOKUP_OBJECT (window1, menu2_uiinfo[4].widget, "r3");
  GLADE_HOOKUP_OBJECT (window1, optionmenu1, "optionmenu1");
  GLADE_HOOKUP_OBJECT (window1, menu1, "menu1");
  GLADE_HOOKUP_OBJECT (window1, menu1_uiinfo[0].widget, "vga1");
  GLADE_HOOKUP_OBJECT (window1, menu1_uiinfo[1].widget, "pal1");
  GLADE_HOOKUP_OBJECT (window1, menu1_uiinfo[2].widget, "sif1");
  GLADE_HOOKUP_OBJECT (window1, menu1_uiinfo[3].widget, "cif1");
  GLADE_HOOKUP_OBJECT (window1, menu1_uiinfo[4].widget, "qpal1");
  GLADE_HOOKUP_OBJECT (window1, menu1_uiinfo[5].widget, "qsif1");
  GLADE_HOOKUP_OBJECT (window1, menu1_uiinfo[6].widget, "qcif1");
  GLADE_HOOKUP_OBJECT (window1, hscale1, "hscale1");
  GLADE_HOOKUP_OBJECT (window1, hscale2, "hscale2");
  GLADE_HOOKUP_OBJECT (window1, hscale3, "hscale3");
  GLADE_HOOKUP_OBJECT (window1, togglebutton2, "togglebutton2");
  GLADE_HOOKUP_OBJECT (window1, image4, "image4");
  GLADE_HOOKUP_OBJECT (window1, togglebutton1, "togglebutton1");
  GLADE_HOOKUP_OBJECT (window1, image3, "image3");
  GLADE_HOOKUP_OBJECT (window1, togglebutton3, "togglebutton3");
  GLADE_HOOKUP_OBJECT (window1, image5, "image5");
  GLADE_HOOKUP_OBJECT (window1, togglebutton4, "togglebutton4");
  GLADE_HOOKUP_OBJECT (window1, image6, "image6");
  GLADE_HOOKUP_OBJECT (window1, togglebutton5, "togglebutton5");
  GLADE_HOOKUP_OBJECT (window1, image7, "image7");

  hidemenus(isjpg);
  if (!isjpg)
  {
	  gtk_menu_set_active(GTK_MENU(gtk_option_menu_get_menu(GTK_OPTION_MENU(optionmenu1))),_DEF_RES);
	  gtk_menu_set_active(GTK_MENU(gtk_option_menu_get_menu(GTK_OPTION_MENU(optionmenu2))),_DEF_COL);
	  gtk_menu_shell_activate_item (GTK_MENU_SHELL(gtk_option_menu_get_menu(GTK_OPTION_MENU(optionmenu1))),menu1_uiinfo[_DEF_RES].widget,TRUE);
	  gtk_menu_shell_activate_item (GTK_MENU_SHELL(gtk_option_menu_get_menu(GTK_OPTION_MENU(optionmenu2))),menu2_uiinfo[_DEF_COL].widget,TRUE);
  }
  else
  {
	  gtk_menu_set_active(GTK_MENU(gtk_option_menu_get_menu(GTK_OPTION_MENU(optionmenu1))),_JPG_DEF_RES);
	  gtk_menu_shell_activate_item (GTK_MENU_SHELL(gtk_option_menu_get_menu(GTK_OPTION_MENU(optionmenu1))),menu1_uiinfo[_JPG_DEF_RES].widget,TRUE);	  
  }
  
  return window1;
}

void setJPGres()
  {
	  gtk_menu_set_active(GTK_MENU(gtk_option_menu_get_menu(GTK_OPTION_MENU(optionmenu1))),_JPG_DEF_RES);
	  gtk_menu_shell_activate_item (GTK_MENU_SHELL(gtk_option_menu_get_menu(GTK_OPTION_MENU(optionmenu1))),menu1_uiinfo[_JPG_DEF_RES].widget,TRUE);	  
  }
