/****************************************************************************
#	 	Gideo: A Small (spca5x) Video Capture utility                       #
# 		Copyright (C) 2004 David Truman	                                    #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 2 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program; if not, write to the Free Software               #
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA #
#                                                                           #
****************************************************************************/



/****************************************************************************
 	Author:		David J Truman 
 	Email:		dave@nyteryda.co.uk
 	version:	0.1
	Date:		16/06/2004
  	Desc:		This Utility is a video capture utility made for use with 
 				spca5x (Sunplus) Linux Kernel Module, the backend is stolen
 				strait out of spcagui written by Michel Xhaard, all i did 
 				was create a gtk/gnome GUI as a replacement for his SDL GUI				
****************************************************************************/


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>


#include "interface.h"
#include "support.h"
#include "spca5xx.h"
#include "spcav4l.h"
#include "utils.h"
#include "globals.h"

int flags, dummy;
struct vdIn myvidIn;

void initialize (const char *videodev, int method);
void processvideo (void);

int
main (int argc, char *argv[])
{
  GtkWidget *window1;

#ifdef ENABLE_NLS
  bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
  textdomain (GETTEXT_PACKAGE);
#endif

  gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE,
                      argc, argv,
                      GNOME_PARAM_APP_DATADIR, PACKAGE_DATA_DIR,
                      NULL);	
	
  const char *videodevice = NULL;
  int grabmethod = 0;
  int i;
  for (i = 1; i < argc; i++)
    {
      /* skip bad arguments */
      if (argv[i] == NULL || *argv[i] == 0 || *argv[i] != '-')
	{
	  continue;
	}
      if (strcmp (argv[i], "-d") == 0)
	{
	  if (i + 1 >= argc)
	    {
	      g_print ("No parameter specified with -d, aborting.\n");
	      exit (1);
	    }
	  videodevice = strdup (argv[i + 1]);
	}
      if (strcmp (argv[i], "-g") == 0)
	{
	  /* Ask for mmap instead default  read */
	  grabmethod = 1;
	}
	if (strcmp (argv[i], "-h") == 0) {
			g_print ("usage: gideo [-h -d -g ] \n");
			g_print ("-h	print this message \n");
			g_print ("-d	/dev/videoX       use videoX device\n");
			g_print ("-g	use read method for grab instead mmap \n");
			exit (0);
	}
    }
  g_print ("Gideo version: %s\n", VERSION );
  if (videodevice == NULL || *videodevice == 0) {
	videodevice = "/dev/video0";
  }
	

  initialize (videodevice, grabmethod);
  window1 = create_window1 ();
  gtk_widget_show (window1);
  

  

  gtk_main ();
  return 0;
}


void
initialize (const char *videodev, int method)
{
  
  /* make room for init data */
  myvidIn.videodevice = NULL;
  myvidIn.cameraname = NULL;
  myvidIn.bridge = NULL;
  myvidIn.videodevice = (char *) realloc (myvidIn.videodevice, 16);
  myvidIn.cameraname = (char *) realloc (myvidIn.cameraname, 32);
  myvidIn.bridge = (char *) realloc (myvidIn.bridge, 9);
  myvidIn.grabMethod = method;
  snprintf (myvidIn.videodevice, 12, "%s", videodev);
  g_print ("video device %s\n", myvidIn.videodevice);
  init_v4l (&myvidIn);
 
  if (myvidIn.cameratype == JPEG)
    isjpg = TRUE;
  else
	isjpg = FALSE;
  
  if ((pipe (fd)) < 0)
    exit (0);
  
  if ((flags = fcntl (fd[0], F_GETFL, dummy)) < 0)
    {
      g_print ("getflags failure %d\n", flags);
      exit (-1);
    }

  if (fcntl (fd[0], F_SETFL, flags | O_NONBLOCK) < 0)
    {
      g_print ("setflags failure\n");
      exit (-1);
    }
  
	// Fork and close appropriate descriptors 
  if ((pid = fork ()) < 0)
    exit (-1);

  if (pid == 0)
    {
      // We are in child
      processvideo ();
      exit (1);
    }

}	


void
init_video (struct spca5xx_frame *myframe)
{
  init_jpeg_decoder ();
  myframe->pictsetting.change = 01;
  myframe->pictsetting.gamma = 3;
  myframe->pictsetting.OffRed = -16;
  myframe->pictsetting.OffBlue = -16;
  myframe->pictsetting.OffGreen = -16;
  myframe->pictsetting.GRed = 256;
  myframe->pictsetting.GBlue = 256;
  myframe->pictsetting.GGreen = 256;
  myframe->pictsetting.force_rgb = 0;
  myframe->cropx1 = myframe->cropx2 = myframe->cropy1 = myframe->cropy2 = 0;
}

int
refresh_screen (struct spca5xx_frame *myframe, struct vdIn *vd,
		SDL_Surface * Screen)
{

  if ((myframe->hdrwidth != vd->hdrwidth) || (myframe->depth != vd->bppIn))
    {
      /* adjust parameters and copy data in -> out */
      myframe->hdrwidth = myframe->width = vd->hdrwidth;
      myframe->hdrheight = myframe->height = vd->hdrheight;
      myframe->depth = vd->bppIn;
      myframe->format = VIDEO_PALETTE_RGB24;
      return -1;
    }
  memcpy (myframe->tmpbuffer, vd->pixTmp, vd->framesizeIn);
  if (SDL_MUSTLOCK (Screen))
    {
      if (SDL_LockSurface (Screen) < 0)
	return -1;
    }
  switch (vd->formatIn)
    {
    case VIDEO_PALETTE_RAW_JPEG:
      {

	//printf("outjpeg w %d h %d\n",myframe->width,myframe->height);
	spca50x_outpicture (myframe);

      }
      break;

    case VIDEO_PALETTE_YUV420P:
      {
	// uncompressed data yuv420P decoder in module
	//printf("outyuv w %d h %d bpp %d\n",myframe->width,myframe->height,vd->bppIn >> 3);
	YUV420toRGB (myframe->tmpbuffer,
		     myframe->data, myframe->width, myframe->height, 0, 0);

      }
      break;
    case VIDEO_PALETTE_RGB565:
    case VIDEO_PALETTE_RGB32:
    case VIDEO_PALETTE_RGB24:
      {
	// uncompressed data. rgb decoder in module simple copy for display.
	//printf("outrgb w %d h %d bpp %d\n",myframe->width,myframe->height,vd->bppIn >> 3);
	memcpy (myframe->data, myframe->tmpbuffer,
		(myframe->width * myframe->height * vd->bppIn) >> 3);
      }
      break;
    default:
      break;
    }
  if (SDL_MUSTLOCK (Screen))
    {
      SDL_UnlockSurface (Screen);
    }
//SDL_Flip(Screen);
  return 0;
}


void
processvideo (void)
{
  SDL_Surface *screen;
  Uint32 video_flags, lib_flags;
  SDL_Event event;
  struct spca5xx_frame *myframe;
  lib_flags = SDL_INIT_VIDEO;
  video_flags = (SDL_HWPALETTE | SDL_DOUBLEBUF);
  int size = 0;
  unsigned char buf[2];

  int len;
  int run = 1;
  int change = 0;
  int pause = 0;
  int avi = 0;
  int fliprgb = 0;
  int photo = 0;

  close (fd[1]);		/* Close the write descriptor */
  if (SDL_Init (lib_flags) < 0)
    {
      fprintf (stderr, "Couldn't initialize SDL: %s\n", SDL_GetError ());
      exit (EXIT_FAILURE);
    }
     putenv ("SDL_VIDEO_CENTERED=1");  
  screen = SDL_SetVideoMode (myvidIn.hdrwidth, myvidIn.hdrheight,
			     0, video_flags);
  SDL_ShowCursor (1);
  SDL_WM_SetCaption ("Gideo Video", "Gideo");
  /* allocate the output frame struct */
  myframe = (struct spca5xx_frame *) malloc (sizeof (struct spca5xx_frame));
  myframe->data = screen->pixels;
  myframe->cameratype = myvidIn.cameratype;
  /* output data definition */
  myframe->width = myvidIn.hdrwidth;
  myframe->height = myvidIn.hdrheight;
  /* output format */
  myframe->format = VIDEO_PALETTE_RGB24;
  /* allocate a big frame buffer */
  size = myframe->width * myframe->height * 4;
  myframe->tmpbuffer = (unsigned char *) malloc (size);

  /* input data maybe a jpeg one */

  if (myvidIn.cameratype == JPEG)
    {
      init_video (myframe);
      myframe->method = 0;
      myvidIn.formatIn = VIDEO_PALETTE_RAW_JPEG;
    }
  else
    {
      myvidIn.formatIn = VIDEO_PALETTE_RGB24;
    }

  setPalette (&myvidIn);


  while (run)
    {
      if (!pause)
	{
	  SpcaGrab (&myvidIn);
	}
      if (photo)
	{
	  photo = 0;
	  getPicture (myvidIn.pixTmp, myvidIn.hdrwidth,
		      myvidIn.hdrheight, myvidIn.formatIn,
		      myvidIn.framesizeIn);
	}

      if (avi)
	{
	  aviWrite (&myvidIn);
	}

      if (refresh_screen (myframe, &myvidIn, screen) < 0)
	{
	  if ((myvidIn.formatIn == VIDEO_PALETTE_YUV420P)
	      || (myvidIn.formatIn == VIDEO_PALETTE_RAW_JPEG))
	    {
	      change = 24;	//RGB24 output
	    }
	  else
	    {
	      change = myframe->depth;
	    }
	 // printf ("setting sdl w %d h %d bpp %d\n", myframe->width,
	 //	  myframe->height, change);
	  screen =
	    SDL_SetVideoMode (myframe->width, myframe->height, change,
			      video_flags);
	  myframe->data = screen->pixels;
	}
      else
	{
	  SDL_Flip (screen);
	}
     
      if ((len = read (fd[0], buf, 2)) > 0)
	{
	 // printf ("get message id %d value %d \n", buf[0], buf[1]);
	  switch (buf[0])
	    {
	    case 1:		/* take a picture only if !avi */
	      if (!avi)
		{
		  /* set picture flag */
		  photo = 1;
		}
	      break;
	    case 2:		/* avi start stop */
	      avi = buf[1];
	      int err = 0;
	      if (avi)
		{
		  err = aviOpen (&myvidIn);
		  if (err < 0)
		    avi = 0;
		}
	      else
		{
		  aviClose ();
		}
	      break;
	    case 3:		/* pause */
	      pause = buf[1];
	      break;
	    case 4:
	      break;
	    case 5:		/* if avi run close avi first */
	      run = 0;
	      break;
	    case 6:		/* maybe flip rgb */
	      fliprgb = buf[1];
	      myvidIn.flipUV = fliprgb;
	      break;
	    case 7:
	      SpcaSetBrightness (&myvidIn, buf[1]);
	      break;
	    case 8:
	      SpcaSetContrast (&myvidIn, buf[1]);
	      break;
	    case 9:
	      SpcaSetColors (&myvidIn, buf[1]);
	      break;
	    case 10:
	      {
		switch (buf[1])
		  {
		  case 1:
		    myvidIn.hdrwidth = 640;
		    myvidIn.hdrheight = 480;
		    break;
		  case 2:
		    myvidIn.hdrwidth = 384;
		    myvidIn.hdrheight = 288;
		    break;
		  case 4:
		    myvidIn.hdrwidth = 352;
		    myvidIn.hdrheight = 288;
		    break;
		  case 8:
		    myvidIn.hdrwidth = 320;
		    myvidIn.hdrheight = 240;
		    break;
		  case 16:
		    myvidIn.hdrwidth = 192;
		    myvidIn.hdrheight = 144;
		    break;
		  case 32:
		    myvidIn.hdrwidth = 176;
		    myvidIn.hdrheight = 144;
		    break;
		  case 64:
		    myvidIn.hdrwidth = 160;
		    myvidIn.hdrheight = 120;
		    break;
		  default:
		    break;
		  }
		changeSize (&myvidIn);
	      }
	      break;
	    case 11:
	      {
		switch (buf[1])
		  {
		  case 1:
		    myvidIn.formatIn = VIDEO_PALETTE_RAW_JPEG;
		    break;
		  case 2:
		    myvidIn.formatIn = VIDEO_PALETTE_YUV420P;
		    break;
		  case 4:
		    myvidIn.formatIn = VIDEO_PALETTE_RGB565;
		    break;
		  case 8:
		    myvidIn.formatIn = VIDEO_PALETTE_RGB24;
		    break;
		  case 16:
		    myvidIn.formatIn = VIDEO_PALETTE_RGB32;
		    break;
		  default:
		    break;
		  }
		setPalette (&myvidIn);
	      }
	      break;
	    default:
	      break;
	    }
	}
      // sleep(1);
    }
  close (fd[0]);		/* Close the file descriptor, we're done */
  sleep (1);
  free (myframe->tmpbuffer);
  free (myframe);
  SDL_Quit ();
}
